'''
update your watchlist by signing into your letterboxd account and exporting your content from the site to a local watchlist.txt file.
'''

#IMPORTS

import os
import os.path

import zipfile
import requests
import pandas as pd

# CONTSTANTS

# letterboxd urls

SIGN_IN_PAGE = "https://letterboxd.com/user/login.do"
EXPORT_PAGE = "https://letterboxd.com/data/export"
REFERER = "https://letterboxd.com/activity/"

# other constants

SIGN_IN_ID_EL = "username"
SIGN_IN_PW_EL = "password"
COOKIE_NAME = "com.xk72.webparts.csrf"
ZIP_FILE = 'data/lb_data.zip'
WATCHLIST_CSV = 'watchlist.csv'
DATA = 'data/'
WATCHLIST_TXT = 'data/watchlist.txt'

MY_PATH = os.path.abspath(os.path.dirname(__file__))

def extract_watchlist():
    '''extracts the watchlist from  the downloaded letterboxd content'''
    zip_file = os.path.join(MY_PATH, ZIP_FILE)
    with zipfile.ZipFile(zip_file, 'r') as zip_ref:
        zip_ref.extract(WATCHLIST_CSV, path=os.path.join(MY_PATH, DATA))
    os.remove(zip_file)

    watchlist_csv = os.path.join(MY_PATH, f"{DATA}{WATCHLIST_CSV}")
    watchlist = list(pd.read_csv(watchlist_csv)['Name'])
    watchlist_txt = os.path.join(MY_PATH, WATCHLIST_TXT)
    with open(watchlist_txt, 'w') as f:
        f.write("\n".join(watchlist))
    os.remove(watchlist_csv)

def download_letterboxd_content():
    '''signs you into letterboxd and then downloads the content'''
    login_payload = {
        SIGN_IN_ID_EL: os.environ['LBXD_USERNAME'],
        SIGN_IN_PW_EL: os.environ['LBXD_PASSWORD']
    }

    with requests.Session() as session:
        session.get(SIGN_IN_PAGE)
        if COOKIE_NAME in session.cookies:
            login_payload['__csrf'] = session.cookies[COOKIE_NAME]

        # sign in
        session.post(SIGN_IN_PAGE, data=login_payload, headers={'referer': REFERER})
        letterboxd_content = session.get(EXPORT_PAGE)

        with open(ZIP_FILE, 'wb') as f:
            f.write(letterboxd_content.content)

def update_watchlist():
    '''updates your watchlist by downloading letterboxd content and then extracting it'''
    download_letterboxd_content()
    extract_watchlist()

if __name__ == "__main__":
    update_watchlist()



# -*- coding: utf-8 -*-
'''
Starts a Firfox headless brower to see if movies on your watchlist are playing
at any of your favorite theaters.

Favorite theaters are taken from a txt file (extracted from "theaters.txt").
These showtimes are compared to a watchlist (extracted from "watchlist.txt")

-samuel mignot-
'''

# ------------------- #
# imports             #
# ------------------- #

import configparser
import os
from os import listdir
from os.path import isfile, join

# internal
import file_helpers

# ------------------- #
# constants           #
# ------------------- #

THIS_DIRECTORY = os.path.abspath(os.path.dirname(__file__))
DATA_DIR = '/data'
THEATERS_DIR = '/theaters'

ERROR_MESSAGE = {
    "theaters_select": "response must be an integer in the range 1-{max_show}",
    "yes_no": "response must be one of the following: 'yes', 'y', 'no', 'n'"
}

def get_theaters():
    mypath = THIS_DIRECTORY+DATA_DIR+THEATERS_DIR
    return [f for f in listdir(mypath) if isfile(join(mypath, f))]

def get_input(question, response_format, error_message):
    ''' loops until a response that is in response_format is met'''
    while True:
        res = input(question)
        if res in response_format:
            return res
        print(error_message)

def set_theaters():
    '''for each movie left filtering, asks the user if they want to watch it and provides showtimes to pick from'''
    theaters_files = get_theaters()
    for i, theater in enumerate(theaters_files, start=1):
        print(f"[{i}]: {theater}")
    res = get_input(
        f"select your theaters file [1-{len(theaters_files)} or n to cancel]: ",
        set(map(str, range(1, len(theaters_files)+1)))|{'n', 'no'},
        ERROR_MESSAGE['theaters_select'].format(max_show=str(len(theaters_files)))
    )
    print()
    if res == 'n':
        return None
    chosen_theaters_file = theaters_files[int(res)-1]
    return chosen_theaters_file

if __name__ == "__main__":
    set_theaters()



'''
run point of application. Calls films to cal module.
Will soon allow configuration and downloading of watchlist.
'''
# external
import configparser
import argparse
import os
import signal

#internal
from film_to_cal import film_to_cal
from set_watchlist import set_watchlist
from set_theaters import set_theaters

# get default settings from settings.ini file
THIS_DIRECTORY = os.path.abspath(os.path.dirname(__file__))
SETTINGS_FILE = "settings.ini"
CRED_FILE = "credentials.json"
config = configparser.ConfigParser()
config.read(f"{THIS_DIRECTORY}/data/{SETTINGS_FILE}")
settings = config['DEFAULT']

# TYPE VERIFICATION
def valid_timezone(timezone_input):
    '''checks whether input string is valid IANA timezone'''
    TIMEZONE_FILE = "timezones.txt"
    with open(f"{THIS_DIRECTORY}/data/{TIMEZONE_FILE}") as f:
        timezones = frozenset([line[:-1] for line in f.readlines()])
    if timezone_input not in timezones:
        msg = f"{timezone_input} is not a IANA timezone"
        raise argparse.ArgumentTypeError(msg)
    return timezone_input

def valid_workdays(workdays_input):
    '''checks whether string is valid workday'''
    msg = f"{workdays_input} is not a valid workdays input. Your input should be a subset of [0,1,2,3,4,5,6] presented as a comma seperated string. ex: 1,2,4."
    try:
        workdays = set(map(int, workdays_input.split(',')))
    except:
        raise argparse.ArgumentTypeError(msg)
    if workdays.issubset(set(range(6))):
        return workdays
    raise argparse.ArgumentTypeError(msg)

def valid_workhours(workhours_input):
    '''checks whether string is valid workhour'''
    msg = f"{workhours_input} is not a valid workhours input. Your input should be two comma-seperated integers between 0 and 23 inclusive. ex. (10,20). These designate the start of work hours and the end of work hours"
    try:
        work_start, work_end = map(int, workhours_input.split(","))
    except:
        raise argparse.ArgumentTypeError(msg)
    if 23>work_start>0 and 23>work_end>0:
        return ((work_start,0), (work_end,0))
    raise argparse.ArgumentTypeError(msg)

def check_for_google_credentials():
    if not os.path.exists(f"{THIS_DIRECTORY}/data/{CRED_FILE}"):
        print(f"WARNING: {THIS_DIRECTORY}/data/{CRED_FILE} does not exist. sceance will be unable to add events to your google calender. To fix this, check the instructions on enabling the google calendar API (https://github.com/sjmignot/sceance#enable-the-google-calendar-api).")

def set_up_argparse():
    '''SET UP ARGPARSE'''
    parser = argparse.ArgumentParser(description='Discover which movies in your watchlist are playing in your favorite theaters. You can change defaults by updating the data/settings.ini file.')

    parser.add_argument('-b', '--browser', default=settings['browser'], choices=['firefox', 'chrome'], help=f"use chrome or firefox headless browser (default: {settings['browser']}).")

    parser.add_argument('-a', '--all-films', action='store_true', help=f"suggest all films instead of filtering based on watchlist.txt.")

    parser.add_argument('-d', '--workdays', type=valid_workdays, default=settings['workdays'], help=f"which days do you work? A comma seperated list of numbers between 0 and 6 (0 is monday, 1 is tueday, ..., 6 is sunday). (default: {settings['workdays']}).")

    parser.add_argument('-w', '--workhours', type=valid_workhours, default=settings['workhours'], help=f"Which hours do you work? Hours are in 24 hour time and formatted: start_hour,end_hour. (default: {settings['workhours']}).")

    parser.add_argument('-t', '--timezone', type=valid_timezone, default=settings['timezone'], help=f"What timezone do you live in? Make sure you provide a valid IANA timezone. This is used by the google calendar api. (default: {settings['timezone']}).")

    parser.add_argument('-v', '--version', action='version', version='1.0.3')

    parser.add_argument('-s', '--set-watchlist', action='store_true', help=f"change the default watchlist from {settings['watchlist']}.")

    parser.add_argument('-c', '--set-theaters', action='store_true', help=f"change the default theaters file from {settings['theaters']}.")

    return vars(parser.parse_args())

def sig_interupt(signum, stack):
    print("\nExiting Program...")
    quit()

def main():
    '''main file for getting showings and savaing them to a google calendar'''
    signal.signal(signal.SIGINT, sig_interupt) #graceful exit on SIGINT
    args = set_up_argparse()
    if args['set_watchlist']:
        watchlist = set_watchlist()
        if watchlist:
            config.set('DEFAULT', 'watchlist', watchlist)
            with open(f"{THIS_DIRECTORY}/data/{SETTINGS_FILE}", 'w') as configfile:
                config.write(configfile)
            print(f"watchlist set to {watchlist}")
        else:
            print(f"watchlist remains unchanged as {settings['watchlist']}")
        return
    if args['set_theaters']:
        theaters =  set_theaters()
        if theaters:
            config.set('DEFAULT', 'theaters', theaters)
            with open(f"{THIS_DIRECTORY}/data/{SETTINGS_FILE}", 'w') as configfile:
                config.write(configfile)
            print(f"theaters file set to {theaters}")
        else:
            print(f"theaters file remains unchanged as {settings['theaters']}")
        return
    args['watchlist'] = settings['watchlist']
    args['theaters'] = settings['theaters']
    check_for_google_credentials()
    film_to_cal(args)

# MAIN FUNCTION: CALLS FILM TO CAL
if __name__ == "__main__":
    main()
    '''
     TODO #
    - add favorite theater
    - update watchlist using letterboxd
    - set busy schedule
    - keep track of films already scheduled and do not prompt if sceance has passed
    '''
